@@ -671,48 +671,19 @@ static void get_socket_name( char* buf, int len )
 
 static gboolean fork_server()
 {
-    const char *server_path = g_find_program_in_path("menu-cached");
     int ret, pid, status;
 
-    if (!server_path)
+    if (!g_file_test (MENUCACHED_LIBEXECDIR "/menu-cached", G_FILE_TEST_IS_EXECUTABLE))
     {
-        g_print("failed to find menu-cached\n");
+        g_error("failed to find menu-cached");
     }
 
-    /* Become a daemon */
+    /* Start daemon */
     pid = fork();
     if (pid == 0)
     {
-        int fd;
-        long open_max;
-        long i;
-
-        /* don't hold open fd opened from the client of the library */
-        open_max = sysconf (_SC_OPEN_MAX);
-        for (i = 0; i < open_max; i++)
-            fcntl (i, F_SETFD, FD_CLOEXEC);
-
-        /* /dev/null for stdin, stdout, stderr */
-        fd = open ("/dev/null", O_RDONLY);
-        if (fd != -1)
-        {
-            dup2 (fd, 0);
-            close (fd);
-        }
-        fd = open ("/dev/null", O_WRONLY);
-        if (fd != -1)
-        {
-            dup2 (fd, 1);
-            dup2 (fd, 2);
-            close (fd);
-        }
-        setsid();
-        if (fork() == 0)
-        {
-            execl( server_path, server_path, NULL);
-            g_print("failed to exec %s\n", server_path);
-        }
-        _exit(0);
+        execl( MENUCACHED_LIBEXECDIR "/menu-cached", MENUCACHED_LIBEXECDIR "/menu-cached", NULL);
+        g_print("failed to exec %s\n", MENUCACHED_LIBEXECDIR "/menu-cached");
     }
 
     /*
@@ -822,7 +793,6 @@ retry:
         if(retries == 0)
         {
             close(fd);
-            unlink(addr.sun_path); /* remove previous socket file */
             fork_server();
             ++retries;
             goto retry;
@@ -854,6 +824,7 @@ MenuCache* register_menu_to_server( const char* menu_name, gboolean re_register
     const char* xdg_data = g_getenv("XDG_DATA_DIRS");
     const char* xdg_cfg_home = g_getenv("XDG_CONFIG_HOME");
     const char* xdg_data_home = g_getenv("XDG_DATA_HOME");
+    const char* xdg_cache_home = g_getenv("XDG_CACHE_HOME");
     char* buf;
     char md5[36];
     char* file_name;
@@ -869,14 +840,17 @@ MenuCache* register_menu_to_server( const char* menu_name, gboolean re_register
         xdg_cfg_home = "";
     if( ! xdg_data_home )
         xdg_data_home = "";
+    if( ! xdg_cache_home )
+        xdg_cache_home = "";
 
     /* get rid of the encoding part of locale name. */
     while( strchr(langs[0], '.') )
         ++langs;
 
-    buf = g_strdup_printf( "REG:%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
+    buf = g_strdup_printf( "REG:%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",
                             menu_name,
                             *langs,
+							xdg_cache_home,
                             xdg_cfg,
                             xdg_prefix,
                             xdg_data,
@@ -912,7 +886,7 @@ MenuCache* register_menu_to_server( const char* menu_name, gboolean re_register
 void unregister_menu_from_server( MenuCache* cache )
 {
     char buf[38];
-    g_snprintf( buf, "UNR:%s\n", cache->md5 );
+    g_snprintf( buf, 38, "UNR:%s\n", cache->md5 );
     write( server_fd, buf, 37 );
 }
 
