@@ -33,8 +33,10 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
+#include <sys/fcntl.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#include <errno.h>
 #include <signal.h>
 #include <utime.h>
 
@@ -51,14 +53,15 @@ typedef struct _Cache
     int n_files;
     char** files;
     GFileMonitor** mons;
-    GFileMonitor* cache_mon;
+    /* GFileMonitor* cache_mon; */
+
+    guint delayed_reload_handler;
 
     /* clients requesting this menu cache */
     GSList* clients;
 }Cache;
 
 static GHashTable* hash = NULL;
-guint delayed_reload_handler = 0;
 
 static void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
                              GFileMonitorEvent evt, Cache* cache );
@@ -78,26 +81,23 @@ static void cache_unref(Cache* cache)
             g_file_monitor_cancel( cache->mons[i] );
             g_object_unref( cache->mons[i] );
         }
+/*
         g_file_monitor_cancel(cache->cache_mon);
         g_object_unref(cache->cache_mon);
-
+*/
         g_free( cache->mons );
         g_strfreev( cache->env );
         g_strfreev( cache->files );
+
+		if( cache->delayed_reload_handler )
+			g_source_remove( cache->delayed_reload_handler );
+
         g_slice_free( Cache, cache );
     }
     else if( cache->n_ref == 1 ) /* the last ref count is held by hash table */
     {
         /* g_debug("menu cache removed from hash"); */
         g_hash_table_remove( hash, cache->md5 );
-        if( g_hash_table_size(hash) == 0 ) /* no menu cache is in use. */
-        {
-            if( delayed_reload_handler )
-            {
-                g_source_remove( delayed_reload_handler );
-                delayed_reload_handler = 0;
-            }
-        }
     }
 }
 
@@ -153,7 +153,8 @@ static void set_env( char* penv, const char* name )
 static void pre_exec( gpointer user_data )
 {
     char** env = (char*)user_data;
-    set_env(*env, "XDG_CONFIG_DIRS");
+    set_env(*env, "XDG_CACHE_HOME");
+    set_env(*++env, "XDG_CONFIG_DIRS");
     set_env(*++env, "XDG_MENU_PREFIX");
     set_env(*++env, "XDG_DATA_DIRS");
     set_env(*++env, "XDG_CONFIG_HOME");
@@ -235,9 +236,10 @@ static gboolean delayed_reload( Cache* cache )
         g_signal_handlers_disconnect_by_func( cache->mons[i], on_file_changed, cache );
         g_object_unref( cache->mons[i] );
     }
+/*
     g_file_monitor_cancel(cache->cache_mon);
     g_object_unref(cache->cache_mon);
-
+*/
     if( ! regenerate_cache( cache->menu_name, cache->lang_name, cache_file,
                             cache->env, &cache->n_files, &cache->files ) )
     {
@@ -256,11 +258,12 @@ static gboolean delayed_reload( Cache* cache )
         g_signal_connect( cache->mons[i], "changed", on_file_changed, cache);
         g_object_unref(gf);
     }
+/*
     gf = g_file_new_for_path( cache_file );
     cache->cache_mon = g_file_monitor_file( gf, 0, NULL, NULL );
     g_signal_connect( cache->cache_mon, "changed", on_file_changed, cache);
     g_object_unref(gf);
-
+*/
     g_free(cache_file);
 
     /* notify the clients that reload is needed. */
@@ -269,7 +272,7 @@ static gboolean delayed_reload( Cache* cache )
         GIOChannel* ch = (GIOChannel*)l->data;
         write(g_io_channel_unix_get_fd(ch), buf, 37 );
     }
-    delayed_reload_handler = 0;
+    cache->delayed_reload_handler = 0;
     return FALSE;
 }
 
@@ -331,7 +334,7 @@ void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
                       GFileMonitorEvent evt, Cache* cache )
 {
     /* g_debug("file %s is changed (%d).", g_file_get_path(gf), evt); */
-    if( mon != cache->cache_mon )
+    /* if( mon != cache->cache_mon ) */
     {
         /* Optimization: Some files in the dir are changed, but it
          * won't affect the content of the menu. So, just omit them,
@@ -402,18 +405,22 @@ void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
 
                     ut.actime = ut.modtime = time(NULL);
                     /* stop monitor of cache file. */
+					/*
                     g_file_monitor_cancel( cache->cache_mon );
                     g_object_unref( cache->cache_mon );
                     cache->cache_mon = NULL;
+					*/
                     /* update the mtime of the cache file. */
                     utime( cache_file, &ut );
-                    /* restart the monitor */
 
-                    gf = g_file_new_for_path( cache_file );
+                    /* restart the monitor */
+                    /* gf = g_file_new_for_path( cache_file ); */
                     g_free(cache_file);
+					/*
                     cache->cache_mon = g_file_monitor_file(gf, 0, NULL, NULL);
                     g_object_unref(gf);
                     g_signal_connect( cache->cache_mon, "changed", G_CALLBACK(on_file_changed), cache);
+					*/
                     g_debug("files are changed, but no re-generation is needed.");
                     return;
                 }
@@ -422,10 +429,10 @@ void on_file_changed( GFileMonitor* mon, GFile* gf, GFile* other,
         }
     }
 
-    if( delayed_reload_handler )
-        g_source_remove(delayed_reload_handler);
+    if( cache->delayed_reload_handler )
+        g_source_remove(cache->delayed_reload_handler);
 
-    delayed_reload_handler = g_timeout_add_seconds_full( G_PRIORITY_LOW, 3, (GSourceFunc)delayed_reload, cache, NULL );
+    cache->delayed_reload_handler = g_timeout_add_seconds_full( G_PRIORITY_LOW, 3, (GSourceFunc)delayed_reload, cache, NULL );
 }
 
 static gboolean cache_file_is_updated( const char* cache_file, int* n_used_files, char** used_files )
@@ -487,9 +494,17 @@ static int create_socket()
     }
     memset(&addr, 0, sizeof(addr));
     addr.sun_family = AF_UNIX;
-
     get_socket_name( addr.sun_path, sizeof( addr.sun_path ) );
-
+    
+    /* remove previous socket file */
+    if (unlink(addr.sun_path) < 0) {
+	if (errno != ENOENT)
+	    g_error("Couldn't remove previous socket file %s", addr.sun_path);
+    }
+    /* remove of previous socket file successful */
+    else
+	g_warning("removed previous socket file %s");
+    
     if(bind(fd, (struct sockaddr *) &addr, sizeof(addr)) < 0)
     {
         g_debug("Failed to bind to socket");
@@ -529,7 +544,7 @@ retry:
         GChecksum *sum;
         int status = 0, n_files, i;
         char *pline = line + 4;
-        char *sep, *menu_name, *lang_name, *cache_file;
+        char *sep, *menu_name, *lang_name, *cache_dir, *cache_file;
         char **files;
         char **env;
         len -= 4;
@@ -537,6 +552,7 @@ retry:
         /* Format of received string, separated by '\t'.
          * Menu Name
          * Language Name
+		  * XDG_CACHE_HOME
          * XDG_CONFIG_DIRS
          * XDG_MENU_PREFIX
          * XDG_DATA_DIRS
@@ -561,8 +577,10 @@ retry:
 
             env = g_strsplit(pline, "\t", 0);
 
-            /* FIXME: should obtain cache dir from client's env */
-            cache_file = g_build_filename(g_get_user_cache_dir(), "menus", md5, NULL );
+			cache_dir = env[0]; /* XDG_CACHE_HOME */
+            /* obtain cache dir from client's env */
+
+            cache_file = g_build_filename(*cache_dir ? cache_dir : g_get_user_cache_dir(), "menus", md5, NULL );
             if( ! cache_file_is_updated(cache_file, &n_files, &files) )
             {
                 /* run menu-cache-gen */
@@ -592,11 +610,12 @@ retry:
                 g_signal_connect( cache->mons[i], "changed", on_file_changed, cache);
                 g_object_unref(gf);
             }
+			/*
             gf = g_file_new_for_path( cache_file );
             cache->cache_mon = g_file_monitor_file( gf, 0, NULL, NULL );
             g_signal_connect( cache->cache_mon, "changed", on_file_changed, cache);
             g_object_unref(gf);
-
+			*/
             g_free(cache_file);
 
             g_hash_table_insert(hash, cache->md5, cache);
@@ -706,12 +725,59 @@ int main(int argc, char** argv)
 {
     GMainLoop* main_loop = g_main_loop_new( NULL, TRUE );
     GIOChannel* ch;
-    int fd;
+    int fd, pid, server_fd;
 
-    fd = create_socket();
-    if( fd < 0 )
+    long open_max;
+    long i;
+
+    server_fd = create_socket();
+
+    if( server_fd < 0 )
         return 1;
 
+    /* Become a daemon */
+
+    if ((pid = fork()) < 0) {
+	g_error("can't fork");
+    }
+    else if (pid != 0) {
+	/* exit parent */
+	exit(0);		
+    }
+
+    /* child (daemon process) != process group leader continues and
+     * becomes a session leader  */
+    setsid();
+
+    /* change working directory to root, so previous working directory
+     * can be unmounted */
+    if (chdir("/") < 0) {
+	g_error("can't change directory to /");
+    }
+
+    open_max = sysconf (_SC_OPEN_MAX);
+    for (i = 0; i < open_max; i++) 
+    {
+	/* don't hold open fd opened besides server socket */
+	if (i != fd)
+	    fcntl (i, F_SETFD, FD_CLOEXEC);
+    }
+    
+    /* /dev/null for stdin, stdout, stderr */
+    fd = open ("/dev/null", O_RDONLY);
+    if (fd != -1)
+    {
+	dup2 (fd, 0);
+	close (fd);
+    }
+    fd = open ("/dev/null", O_WRONLY);
+    if (fd != -1)
+    {
+	dup2 (fd, 1);
+	dup2 (fd, 2);
+	close (fd);
+    }
+
     signal(SIGHUP, terminate);
     signal(SIGINT, terminate);
     signal(SIGQUIT, terminate);
@@ -719,7 +785,7 @@ int main(int argc, char** argv)
     signal(SIGKILL, terminate);
     signal(SIGPIPE, SIG_IGN);
 
-    ch = g_io_channel_unix_new(fd);
+    ch = g_io_channel_unix_new(server_fd);
     if(!ch)
         return 1;
     g_io_channel_set_close_on_unref(ch, TRUE);
